#include <cstdlib>
#include <cmath>

#include "newton.h"


Newton::Newton(Fct *f, double eps, double xmin, double xmax, int maxiter)
{
	if (xmax <= xmin) throw "bornes invalides";

	f_ = f;
	eps_ = eps;
	xmin_ = xmin;
	xmax_ = xmax;
	maxiter_ = maxiter;
	abscisse_ = NULL;
	res_ = NULL;
	N_ = 0;
}

double Newton::operator()(double x0)
{
	if ((x0 < xmin_) || (x0 > xmax_)) throw "condition initiale invalide";

	double res = x0;
	int i = 0;

	while ((fabs(f_->fct(res) / f_->fct(x0)) > eps_) && (i <= maxiter_))
	{
		res -= f_->fct(res) / f_->der(res);
		i++;
	}

	if (i > maxiter_) throw "pas de convergence";

	return res;
}

void Newton::discretisation(int N)
{
	if (N <= 0) throw "mauvais nombre de discretisation";

	abscisse_ = new double[N];
	if (!abscisse_) throw "echec de l'allocation de memoire";
	res_ = new double[N];
	if (!res_) throw "echec de l'allocation de memoire";
	N_ = N;

	for (int i = 0; i < N; i++)
	{
		abscisse_[i] = xmin_ + i * (xmax_ - xmin_) / N;
		res_[i] = operator()(abscisse_[i]);
	}
}
